# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/orestes/Devel/Software/pycirkuit/pycirkuit/ui/configdialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ConfigDialog(object):
    def setupUi(self, ConfigDialog):
        ConfigDialog.setObjectName("ConfigDialog")
        ConfigDialog.resize(899, 561)
        ConfigDialog.setMinimumSize(QtCore.QSize(800, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(ConfigDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.listWidget = QtWidgets.QListWidget(ConfigDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidget.sizePolicy().hasHeightForWidth())
        self.listWidget.setSizePolicy(sizePolicy)
        self.listWidget.setMaximumSize(QtCore.QSize(128, 16777215))
        self.listWidget.setAutoFillBackground(False)
        self.listWidget.setStyleSheet("padding-top: 10px; \n"
"background-color: rgb(255, 255, 255);")
        self.listWidget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.listWidget.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.listWidget.setIconSize(QtCore.QSize(48, 48))
        self.listWidget.setTextElideMode(QtCore.Qt.ElideNone)
        self.listWidget.setFlow(QtWidgets.QListView.LeftToRight)
        self.listWidget.setProperty("isWrapping", True)
        self.listWidget.setViewMode(QtWidgets.QListView.IconMode)
        self.listWidget.setWordWrap(True)
        self.listWidget.setObjectName("listWidget")
        item = QtWidgets.QListWidgetItem()
        item.setTextAlignment(QtCore.Qt.AlignCenter)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/pixmaps/CfgGeneral"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.listWidget.addItem(item)
        item = QtWidgets.QListWidgetItem()
        item.setTextAlignment(QtCore.Qt.AlignCenter)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/pixmaps/CfgExportImage"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon1)
        item.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
        self.listWidget.addItem(item)
        self.horizontalLayout.addWidget(self.listWidget)
        self.stackedWidget = QtWidgets.QStackedWidget(ConfigDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(4)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.stackedWidget.sizePolicy().hasHeightForWidth())
        self.stackedWidget.setSizePolicy(sizePolicy)
        self.stackedWidget.setBaseSize(QtCore.QSize(0, 0))
        self.stackedWidget.setFrameShape(QtWidgets.QFrame.Panel)
        self.stackedWidget.setFrameShadow(QtWidgets.QFrame.Raised)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page1 = QtWidgets.QWidget()
        self.page1.setObjectName("page1")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.page1)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(self.page1)
        self.groupBox.setMinimumSize(QtCore.QSize(0, 100))
        self.groupBox.setAlignment(QtCore.Qt.AlignCenter)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.cmPath = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cmPath.sizePolicy().hasHeightForWidth())
        self.cmPath.setSizePolicy(sizePolicy)
        self.cmPath.setClearButtonEnabled(True)
        self.cmPath.setObjectName("cmPath")
        self.horizontalLayout_3.addWidget(self.cmPath)
        self.toolButtonCMPath = QtWidgets.QToolButton(self.groupBox)
        self.toolButtonCMPath.setMinimumSize(QtCore.QSize(120, 0))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/Open"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toolButtonCMPath.setIcon(icon2)
        self.toolButtonCMPath.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolButtonCMPath.setObjectName("toolButtonCMPath")
        self.horizontalLayout_3.addWidget(self.toolButtonCMPath)
        self.verticalLayout_2.addWidget(self.groupBox)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        self.verticalLayout_2.addItem(spacerItem)
        self.groupBox_2 = QtWidgets.QGroupBox(self.page1)
        self.groupBox_2.setMinimumSize(QtCore.QSize(0, 100))
        self.groupBox_2.setAlignment(QtCore.Qt.AlignCenter)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.templateFile = QtWidgets.QLineEdit(self.groupBox_2)
        self.templateFile.setClearButtonEnabled(True)
        self.templateFile.setObjectName("templateFile")
        self.horizontalLayout_4.addWidget(self.templateFile)
        self.toolButtonTemplatePath = QtWidgets.QToolButton(self.groupBox_2)
        self.toolButtonTemplatePath.setMinimumSize(QtCore.QSize(120, 0))
        self.toolButtonTemplatePath.setIcon(icon2)
        self.toolButtonTemplatePath.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolButtonTemplatePath.setObjectName("toolButtonTemplatePath")
        self.horizontalLayout_4.addWidget(self.toolButtonTemplatePath)
        self.verticalLayout_2.addWidget(self.groupBox_2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.stackedWidget.addWidget(self.page1)
        self.page2 = QtWidgets.QWidget()
        self.page2.setObjectName("page2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.page2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox_vector = QtWidgets.QGroupBox(self.page2)
        self.groupBox_vector.setMinimumSize(QtCore.QSize(0, 110))
        self.groupBox_vector.setObjectName("groupBox_vector")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.groupBox_vector)
        self.horizontalLayout_6.setSpacing(20)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_2 = QtWidgets.QLabel(self.groupBox_vector)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(3)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setWordWrap(True)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_6.addWidget(self.label_2)
        self.frame = QtWidgets.QFrame(self.groupBox_vector)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.exportTIKZ = QtWidgets.QCheckBox(self.frame)
        self.exportTIKZ.setChecked(True)
        self.exportTIKZ.setObjectName("exportTIKZ")
        self.verticalLayout_5.addWidget(self.exportTIKZ)
        self.exportSVG = QtWidgets.QCheckBox(self.frame)
        self.exportSVG.setObjectName("exportSVG")
        self.verticalLayout_5.addWidget(self.exportSVG)
        self.exportPDF = QtWidgets.QCheckBox(self.frame)
        self.exportPDF.setObjectName("exportPDF")
        self.verticalLayout_5.addWidget(self.exportPDF)
        self.exportPNG = QtWidgets.QCheckBox(self.frame)
        self.exportPNG.setChecked(False)
        self.exportPNG.setObjectName("exportPNG")
        self.verticalLayout_5.addWidget(self.exportPNG)
        self.exportJPEG = QtWidgets.QCheckBox(self.frame)
        self.exportJPEG.setObjectName("exportJPEG")
        self.verticalLayout_5.addWidget(self.exportJPEG)
        self.horizontalLayout_6.addWidget(self.frame)
        self.verticalLayout_3.addWidget(self.groupBox_vector)
        spacerItem2 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        self.verticalLayout_3.addItem(spacerItem2)
        self.groupBox_rasterParam = QtWidgets.QGroupBox(self.page2)
        self.groupBox_rasterParam.setMinimumSize(QtCore.QSize(0, 130))
        self.groupBox_rasterParam.setObjectName("groupBox_rasterParam")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.groupBox_rasterParam)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.labelDPI = QtWidgets.QLabel(self.groupBox_rasterParam)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(5)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelDPI.sizePolicy().hasHeightForWidth())
        self.labelDPI.setSizePolicy(sizePolicy)
        self.labelDPI.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.labelDPI.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.labelDPI.setObjectName("labelDPI")
        self.horizontalLayout_8.addWidget(self.labelDPI)
        self.exportDPI = QtWidgets.QSpinBox(self.groupBox_rasterParam)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.exportDPI.sizePolicy().hasHeightForWidth())
        self.exportDPI.setSizePolicy(sizePolicy)
        self.exportDPI.setFrame(True)
        self.exportDPI.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.exportDPI.setMaximum(3000)
        self.exportDPI.setSingleStep(10)
        self.exportDPI.setProperty("value", 150)
        self.exportDPI.setObjectName("exportDPI")
        self.horizontalLayout_8.addWidget(self.exportDPI)
        self.verticalLayout_6.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.labelQuality = QtWidgets.QLabel(self.groupBox_rasterParam)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(5)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelQuality.sizePolicy().hasHeightForWidth())
        self.labelQuality.setSizePolicy(sizePolicy)
        self.labelQuality.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.labelQuality.setObjectName("labelQuality")
        self.horizontalLayout_7.addWidget(self.labelQuality)
        self.exportQuality = QtWidgets.QSpinBox(self.groupBox_rasterParam)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.exportQuality.sizePolicy().hasHeightForWidth())
        self.exportQuality.setSizePolicy(sizePolicy)
        self.exportQuality.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.exportQuality.setMaximum(100)
        self.exportQuality.setProperty("value", 80)
        self.exportQuality.setObjectName("exportQuality")
        self.horizontalLayout_7.addWidget(self.exportQuality)
        self.verticalLayout_6.addLayout(self.horizontalLayout_7)
        self.verticalLayout_3.addWidget(self.groupBox_rasterParam)
        spacerItem3 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.stackedWidget.addWidget(self.page2)
        self.horizontalLayout.addWidget(self.stackedWidget)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.buttonBox = QtWidgets.QDialogButtonBox(ConfigDialog)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setCenterButtons(False)
        self.buttonBox.setObjectName("buttonBox")
        self.horizontalLayout_2.addWidget(self.buttonBox)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.labelDPI.setBuddy(self.exportDPI)
        self.labelQuality.setBuddy(self.exportQuality)

        self.retranslateUi(ConfigDialog)
        self.stackedWidget.setCurrentIndex(1)
        self.listWidget.currentRowChanged['int'].connect(self.stackedWidget.setCurrentIndex)
        self.buttonBox.accepted.connect(ConfigDialog.accept)
        self.buttonBox.rejected.connect(ConfigDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(ConfigDialog)
        ConfigDialog.setTabOrder(self.listWidget, self.cmPath)
        ConfigDialog.setTabOrder(self.cmPath, self.toolButtonCMPath)
        ConfigDialog.setTabOrder(self.toolButtonCMPath, self.templateFile)
        ConfigDialog.setTabOrder(self.templateFile, self.toolButtonTemplatePath)
        ConfigDialog.setTabOrder(self.toolButtonTemplatePath, self.exportTIKZ)
        ConfigDialog.setTabOrder(self.exportTIKZ, self.exportPDF)
        ConfigDialog.setTabOrder(self.exportPDF, self.exportDPI)
        ConfigDialog.setTabOrder(self.exportDPI, self.exportQuality)

    def retranslateUi(self, ConfigDialog):
        _translate = QtCore.QCoreApplication.translate
        ConfigDialog.setWindowTitle(_translate("ConfigDialog", "Config Dialog"))
        __sortingEnabled = self.listWidget.isSortingEnabled()
        self.listWidget.setSortingEnabled(False)
        item = self.listWidget.item(0)
        item.setText(_translate("ConfigDialog", "General"))
        item = self.listWidget.item(1)
        item.setText(_translate("ConfigDialog", "Exporting"))
        self.listWidget.setSortingEnabled(__sortingEnabled)
        self.groupBox.setTitle(_translate("ConfigDialog", "Circuit Macros Location", "Group box title"))
        self.cmPath.setPlaceholderText(_translate("ConfigDialog", "Enter Path Name"))
        self.toolButtonCMPath.setText(_translate("ConfigDialog", "Choose...", "Button text"))
        self.groupBox_2.setTitle(_translate("ConfigDialog", "LaTeX Template File", "Group box title"))
        self.templateFile.setPlaceholderText(_translate("ConfigDialog", "Enter Path Name"))
        self.toolButtonTemplatePath.setText(_translate("ConfigDialog", "Choose...", "Button text"))
        self.groupBox_vector.setTitle(_translate("ConfigDialog", "Image formats to export", "Group box title"))
        self.label_2.setText(_translate("ConfigDialog", "Choose which formats will be saved when the button \"Export\" is pressed:"))
        self.exportTIKZ.setText(_translate("ConfigDialog", "&TIKZ"))
        self.exportSVG.setText(_translate("ConfigDialog", "SVG"))
        self.exportPDF.setText(_translate("ConfigDialog", "PD&F"))
        self.exportPNG.setText(_translate("ConfigDialog", "&PNG (raster)"))
        self.exportJPEG.setText(_translate("ConfigDialog", "&JPEG (raster)"))
        self.groupBox_rasterParam.setTitle(_translate("ConfigDialog", "Raster image export parameters", "Group box title"))
        self.labelDPI.setText(_translate("ConfigDialog", "Ras&ter image resolution in dots per inch (dpi):"))
        self.exportDPI.setSuffix(_translate("ConfigDialog", " dpi", "Suffix in spin box (translate to acronym)"))
        self.labelQuality.setText(_translate("ConfigDialog", "Image &quality for lossy formats like jpeg (percent):"))
        self.exportQuality.setSuffix(_translate("ConfigDialog", " %", "Spin box suffix"))


from pycirkuit.resources import resources_rc


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ConfigDialog = QtWidgets.QDialog()
    ui = Ui_ConfigDialog()
    ui.setupUi(ConfigDialog)
    ConfigDialog.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""
Application core functionality
"""
# Copyright (C) 2018-2019 Orestes Mas
# Copyright (C) 2019 Aniol Marti
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
import os
import shutil
from enum import Enum

# Third-party imports
from PyQt5.QtCore import QCoreApplication, QObject, QTemporaryDir, QSettings

# Local imports
import pycirkuit
from pycirkuit import Option
from pycirkuit.exceptions import *
from pycirkuit.tools.m4 import ToolM4
from pycirkuit.tools.dpic import ToolDpic
from pycirkuit.tools.pdflatex import ToolPdfLaTeX
from pycirkuit.tools.pdftopng import ToolPdfToPng
from pycirkuit.tools.pdftojpg import ToolPdfToJpeg
from pycirkuit.tools.circuitmacrosmanager import CircuitMacrosManager

# Translation function
_translate = QCoreApplication.translate

class Overwrite(Enum):
    UNSET = 0
    YES = 1
    ALL = 2
    NO = 3
    NEVER =4
    
class PyCirkuitProcessor(QObject):
    TMP_FILE_BASENAME= "cirkuit_tmp"
    def __init__(self):
        self.environmentOk = False
        super().__init__()
        self.overwrite = Overwrite.UNSET
        # Save current work dir and change into a temporary one
        self.savedWD = os.getcwd()
        pycirkuit.__tmpDir__ = QTemporaryDir()
        os.chdir(pycirkuit.__tmpDir__.path())
  
    def __del__(self):
        # Restore working dir and remove temporary dir
        os.chdir(self.savedWD)
        pycirkuit.__tmpDir__.remove()

    def _check_latex_template(self):
        settings = QSettings()
        template = settings.value("General/templatePath",  "")
        if os.path.exists(template):
            with open(template, 'r') as t:
                templateCode = t.read()
                if "%%SOURCE%%" in templateCode:
                    return True
                else:
                    raise PyCktLatexTemplateError(_translate("CommandLine", "Found an invalid LaTeX template.", "Error message."))
        else:
            raise PyCktLatexTemplateError(_translate("CommandLine", "LaTeX template not found.", "Error message."))

    def _check_programs(self):
        # Dictionary using a class as index and a class instance as value
        self.extTools = {
            ToolM4: ToolM4(),
            ToolDpic: ToolDpic(), 
            ToolPdfLaTeX: ToolPdfLaTeX(), 
            ToolPdfToPng: ToolPdfToPng(), 
            ToolPdfToJpeg: ToolPdfToJpeg()
        }

    def _check_circuit_macros(self):
        cmMgr = CircuitMacrosManager()
        if not cmMgr.check_installed():
            raise PyCktCMNotFoundError(_translate("CommandLine", "Cannot find the Circuit Macros!",  "Command line error message."))

    def _askUser(self):
        print(_translate("CommandLine-UserInput1",  "\nThe destination file already exists.", "Command line message. THE STARTING NEWLINE CHARACTER (\n) IS IMPORTANT."))
        question = _translate("CommandLine-UserInput1",
            "Would you like to overwrite it? ([y]es | [n]o | yes to [a]ll | ne[v]er): ",
            "WARNING!! Critical translation. You should translate this message to your language, enclosing into brackets one single DIFFERENT character for each option, and translate accordingly the characters in the next message.")
        answerYes = _translate("CommandLine-UserInput1",
            "y", 
            "WARNING!! Critical translation. This char must match one of those of the message 'Would you like to overwrite it?'")
        answerNo = _translate("CommandLine-UserInput1",
            "n", 
            "WARNING!! Critical translation. This char must match one of those of the message 'Would you like to overwrite it?'")
        answerAll = _translate("CommandLine-UserInput1",
            "a", 
            "WARNING!! Critical translation. This char must match one of those of the message 'Would you like to overwrite it?'")
        answerNever = _translate("CommandLine-UserInput1",
            "v", 
            "WARNING!! Critical translation. This char must match one of those of the message 'Would you like to overwrite it?'")
        answers = {
            answerYes: Overwrite.YES,
            answerNo: Overwrite.NO,
            answerAll: Overwrite.ALL,
            answerNever: Overwrite.NEVER
        }
        while True:
            answer = input(question)
            if answer.lower() not in answers:
                continue
            else:
                return answers[answer]

    def checkEnvironment(self):
        self._check_programs()
        self._check_circuit_macros()
        self._check_latex_template()
        self.environmentOk = True

    def beginProcessing(self, src):
        # 1) Check if all tools are installed
        if not self.environmentOk:
            self.checkEnvironment()
        # 2) Set all formats to "ungenerated" state
        self.picExists = self.tikzExists = self.svgExists = self.pdfExists = self.pngExists = self.jpegExists = False
        # Copy source file into temporary file
        self.sourceFile = src
        dst = PyCirkuitProcessor.TMP_FILE_BASENAME + ".ckt"
        shutil.copy(src, dst)
        print(os.path.basename(self.sourceFile), end="")

    def requestResult(self, option, dstDir="", overwrite=Overwrite.UNSET, dpi=None, quality=None):
        if option == Option.PNG:
            extension = "png"
            startPoint = self.toPng
            formatToCheck = self.pngExists
        elif option == Option.JPEG:
            extension = "jpeg"
            startPoint = self.toJpeg
            formatToCheck = self.jpegExists
        elif option == Option.PDF:
            extension = "pdf"
            startPoint = self.toPdf
            formatToCheck = self.pdfExists
        elif option == Option.SVG:
            extension = "svg"
            startPoint = self.toSvg
            formatToCheck = self.svgExists
        elif option == Option.TIKZ:
            extension = "tikz"
            startPoint = self.toTikz
            formatToCheck = self.tikzExists
        if (overwrite == Overwrite.ALL) or (overwrite == Overwrite.NEVER):
            self.overwrite = overwrite
        src = os.path.join(pycirkuit.__tmpDir__.path(), PyCirkuitProcessor.TMP_FILE_BASENAME) + os.extsep + extension
        if dstDir:
            filename = os.path.basename(self.sourceFile).rpartition(os.extsep)[0]
            dst = os.path.join(dstDir, filename) + os.extsep + extension
        else:
            dst = self.sourceFile.rpartition(os.extsep)[0] + os.extsep + extension
        if os.path.isfile(dst):
            if self.overwrite == Overwrite.UNSET:
                self.overwrite = self._askUser()
            if (self.overwrite == Overwrite.NO) or (self.overwrite == Overwrite.NEVER):
                if (self.overwrite == Overwrite.NO):
                    self.overwrite = Overwrite.UNSET
                return
            if (self.overwrite == Overwrite.YES):
                self.overwrite = Overwrite.UNSET
        # Let's recurse
        if not formatToCheck:
            startPoint(dpi, quality)
        # Si el font NO existeix, demanar crear-lo. Això implica
        shutil.copy(src, dst)

    def toPng(self, dpi, q=None):
        if not self.pngExists:
            self.toPdf()
            self.extTools[ToolPdfToPng].execute(PyCirkuitProcessor.TMP_FILE_BASENAME, resolution=dpi)
            print(" -> PNG ({} dpi)".format(dpi), end="")
            self.pngExists = True
        return True

    def toJpeg(self, dpi, q):
        if not self.jpegExists:
            self.toPdf()
            self.extTools[ToolPdfToJpeg].execute(PyCirkuitProcessor.TMP_FILE_BASENAME, resolution=dpi, quality=q)
            print(" -> JPEG ({dpi} dpi, {q}% quality)".format(dpi=dpi, q=q), end="")
            self.jpegExists = True
        return True

    def toPdf(self, dpi=None, q=None):
        if not self.pdfExists:
            self.toTikz()
            self.extTools[ToolPdfLaTeX].execute(PyCirkuitProcessor.TMP_FILE_BASENAME)
            print(" -> PDF", end="")
            self.pdfExists = True
        return True

    def toSvg(self, dpi=None, q=None):
        if not self.svgExists:
            self.toPic()
            self.extTools[ToolDpic].execute(PyCirkuitProcessor.TMP_FILE_BASENAME, outputType=Option.SVG)
            print(" -> SVG", end="")
            self.svgExists = True
        return True

    def toTikz(self, dpi=None, q=None):
        if not self.tikzExists:
            self.toPic()
            self.extTools[ToolDpic].execute(PyCirkuitProcessor.TMP_FILE_BASENAME, outputType=Option.TIKZ)
            print(" -> TIKZ", end="")
            self.tikzExists = True
        return True

    def toPic(self, dpi=None, q=None):
        if not self.picExists:
            self.extTools[ToolM4].execute(PyCirkuitProcessor.TMP_FILE_BASENAME)
            print(" -> PIC", end="")
            self.picExists = True
        return True



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/orestes/Devel/Software/pycirkuit/pycirkuit/ui/aboutdialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutDialog(object):
    def setupUi(self, AboutDialog):
        AboutDialog.setObjectName("AboutDialog")
        AboutDialog.resize(750, 680)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AboutDialog.sizePolicy().hasHeightForWidth())
        AboutDialog.setSizePolicy(sizePolicy)
        AboutDialog.setMinimumSize(QtCore.QSize(750, 680))
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        AboutDialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(AboutDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem)
        self.label = QtWidgets.QLabel(AboutDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setMinimumSize(QtCore.QSize(0, 49))
        self.label.setAutoFillBackground(True)
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/pixmaps/AppLabel"))
        self.label.setScaledContents(False)
        self.label.setTextInteractionFlags(QtCore.Qt.NoTextInteraction)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        spacerItem1 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem1)
        self.tabWidget = QtWidgets.QTabWidget(AboutDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(5)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        self.tabWidget.setFont(font)
        self.tabWidget.setAutoFillBackground(False)
        self.tabWidget.setStyleSheet("padding-left:5; padding-top:10; padding-bottom:10; padding-right:5; background-color: rgb(255, 255, 255);")
        self.tabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabWidget.setObjectName("tabWidget")
        self.tabAbout = QtWidgets.QWidget()
        self.tabAbout.setObjectName("tabAbout")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.tabAbout)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.textAbout = QtWidgets.QTextBrowser(self.tabAbout)
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        self.textAbout.setFont(font)
        self.textAbout.setStyleSheet("padding-left:5; padding-top:10; padding-bottom:10; padding-right:5; background-color: rgb(255, 255, 255);")
        self.textAbout.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.textAbout.setOpenExternalLinks(True)
        self.textAbout.setObjectName("textAbout")
        self.horizontalLayout.addWidget(self.textAbout)
        self.tabWidget.addTab(self.tabAbout, "")
        self.tabVersion = QtWidgets.QWidget()
        self.tabVersion.setObjectName("tabVersion")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.tabVersion)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.textVersion = QtWidgets.QTextBrowser(self.tabVersion)
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        self.textVersion.setFont(font)
        self.textVersion.setStyleSheet("padding-left:5; padding-top:10; padding-bottom:10; padding-right:5; background-color: rgb(255, 255, 255);")
        self.textVersion.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.textVersion.setOpenExternalLinks(True)
        self.textVersion.setObjectName("textVersion")
        self.horizontalLayout_4.addWidget(self.textVersion)
        self.tabWidget.addTab(self.tabVersion, "")
        self.tabAuthor = QtWidgets.QWidget()
        self.tabAuthor.setObjectName("tabAuthor")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tabAuthor)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.textAuthor = QtWidgets.QTextBrowser(self.tabAuthor)
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        self.textAuthor.setFont(font)
        self.textAuthor.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.textAuthor.setAutoFormatting(QtWidgets.QTextEdit.AutoNone)
        self.textAuthor.setAcceptRichText(True)
        self.textAuthor.setOpenExternalLinks(True)
        self.textAuthor.setObjectName("textAuthor")
        self.horizontalLayout_3.addWidget(self.textAuthor)
        self.tabWidget.addTab(self.tabAuthor, "")
        self.tabGPL = QtWidgets.QWidget()
        self.tabGPL.setObjectName("tabGPL")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tabGPL)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.textLicense = QtWidgets.QTextBrowser(self.tabGPL)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.textLicense.sizePolicy().hasHeightForWidth())
        self.textLicense.setSizePolicy(sizePolicy)
        self.textLicense.setMinimumSize(QtCore.QSize(0, 220))
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        self.textLicense.setFont(font)
        self.textLicense.setAutoFillBackground(False)
        self.textLicense.setStyleSheet("padding-left:5; padding-top:10; padding-bottom:10; padding-right:5; background-color: rgb(255, 255, 255);")
        self.textLicense.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.textLicense.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textLicense.setOpenExternalLinks(True)
        self.textLicense.setObjectName("textLicense")
        self.verticalLayout_2.addWidget(self.textLicense)
        self.line = QtWidgets.QFrame(self.tabGPL)
        self.line.setMinimumSize(QtCore.QSize(0, 1))
        self.line.setLineWidth(2)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.textGPL = QtWidgets.QTextBrowser(self.tabGPL)
        font = QtGui.QFont()
        font.setFamily("Sans Serif")
        self.textGPL.setFont(font)
        self.textGPL.setStyleSheet("padding-left:5; padding-top:10; padding-bottom:10; padding-right:5; background-color: rgb(255, 255, 255);")
        self.textGPL.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.textGPL.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.textGPL.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.textGPL.setDocumentTitle("GNU General Public License v3.0 - GNU Project - Free Software Foundation (FSF)")
        self.textGPL.setHtml("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><title>GNU General Public License v3.0 - GNU Project - Free Software Foundation (FSF)</title><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:12pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:14px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">GNU GENERAL PUBLIC LICENSE</span></p>\n"
"<p align=\"center\" style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Version 3, 29 June 2007</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Copyright © 2007 Free Software Foundation, Inc. &lt;</span><a href=\"https://fsf.org/\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt; text-decoration: underline; color:#2980b9;\">https://fsf.org/</span></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">&gt;</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Everyone is permitted to copy and distribute verbatim copies of this license document, but changing it is not allowed.</span></p>\n"
"<p style=\" margin-top:14px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"preamble\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">P</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">reamble</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The GNU General Public License is a free, copyleft license forsoftware and other kinds of works.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The licenses for most software and other practical works are designedto take away your freedom to share and change the works.  By contrast,the GNU General Public License is intended to guarantee your freedom toshare and change all versions of a program--to make sure it remains freesoftware for all its users.  We, the Free Software Foundation, use theGNU General Public License for most of our software; it applies also toany other work released this way by its authors.  You can apply it toyour programs, too.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">When we speak of free software, we are referring to freedom, notprice.  Our General Public Licenses are designed to make sure that youhave the freedom to distribute copies of free software (and charge forthem if you wish), that you receive source code or can get it if youwant it, that you can change the software or use pieces of it in newfree programs, and that you know you can do these things.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">To protect your rights, we need to prevent others from denying youthese rights or asking you to surrender the rights.  Therefore, you havecertain responsibilities if you distribute copies of the software, or ifyou modify it: responsibilities to respect the freedom of others.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">For example, if you distribute copies of such a program, whethergratis or for a fee, you must pass on to the recipients the samefreedoms that you received.  You must make sure that they, too, receiveor can get the source code.  And you must show them these terms so theyknow their rights.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Developers that use the GNU GPL protect your rights with two steps:(1) assert copyright on the software, and (2) offer you this Licensegiving you legal permission to copy, distribute and/or modify it.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">For the developers\' and authors\' protection, the GPL clearly explainsthat there is no warranty for this free software.  For both users\' andauthors\' sake, the GPL requires that modified versions be marked aschanged, so that their problems will not be attributed erroneously toauthors of previous versions.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Some devices are designed to deny users access to install or runmodified versions of the software inside them, although the manufacturercan do so.  This is fundamentally incompatible with the aim ofprotecting users\' freedom to change the software.  The systematicpattern of such abuse occurs in the area of products for individuals touse, which is precisely where it is most unacceptable.  Therefore, wehave designed this version of the GPL to prohibit the practice for thoseproducts.  If such problems arise substantially in other domains, westand ready to extend this provision to those domains in future versionsof the GPL, as needed to protect the freedom of users.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Finally, every program is threatened constantly by software patents.States should not allow patents to restrict development and use ofsoftware on general-purpose computers, but in those that do, we wish toavoid the special danger that patents applied to a free program couldmake it effectively proprietary.  To prevent this, the GPL assures thatpatents cannot be used to render the program non-free.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The precise terms and conditions for copying, distribution andmodification follow.</span></p>\n"
"<p style=\" margin-top:14px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"terms\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">T</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">ERMS AND CONDITIONS</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section0\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">0</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Definitions.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">“This License” refers to version 3 of the GNU General Public License.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">“Copyright” also means copyright-like laws that apply to other kinds ofworks, such as semiconductor masks. </span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">“The Program” refers to any copyrightable work licensed under thisLicense.  Each licensee is addressed as “you”.  “Licensees” and“recipients” may be individuals or organizations.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">To “modify” a work means to copy from or adapt all or part of the workin a fashion requiring copyright permission, other than the making of anexact copy.  The resulting work is called a “modified version” of theearlier work or a work “based on” the earlier work.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">A “covered work” means either the unmodified Program or a work basedon the Program.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">To “propagate” a work means to do anything with it that, withoutpermission, would make you directly or secondarily liable forinfringement under applicable copyright law, except executing it on acomputer or modifying a private copy.  Propagation includes copying,distribution (with or without modification), making available to thepublic, and in some countries other activities as well.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">To “convey” a work means any kind of propagation that enables otherparties to make or receive copies.  Mere interaction with a user througha computer network, with no transfer of a copy, is not conveying.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">An interactive user interface displays “Appropriate Legal Notices”to the extent that it includes a convenient and prominently visiblefeature that (1) displays an appropriate copyright notice, and (2)tells the user that there is no warranty for the work (except to theextent that warranties are provided), that licensees may convey thework under this License, and how to view a copy of this License.  Ifthe interface presents a list of user commands or options, such as amenu, a prominent item in the list meets this criterion.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section1\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Source Code.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The “source code” for a work means the preferred form of the workfor making modifications to it.  “Object code” means any non-sourceform of a work.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">A “Standard Interface” means an interface that either is an officialstandard defined by a recognized standards body, or, in the case ofinterfaces specified for a particular programming language, one thatis widely used among developers working in that language.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The “System Libraries” of an executable work include anything, otherthan the work as a whole, that (a) is included in the normal form ofpackaging a Major Component, but which is not part of that MajorComponent, and (b) serves only to enable use of the work with thatMajor Component, or to implement a Standard Interface for which animplementation is available to the public in source code form.  A“Major Component”, in this context, means a major essential component(kernel, window system, and so on) of the specific operating system(if any) on which the executable work runs, or a compiler used toproduce the work, or an object code interpreter used to run it.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The “Corresponding Source” for a work in object code form means allthe source code needed to generate, install, and (for an executablework) run the object code and to modify the work, including scripts tocontrol those activities.  However, it does not include the work\'sSystem Libraries, or general-purpose tools or generally available freeprograms which are used unmodified in performing those activities butwhich are not part of the work.  For example, Corresponding Sourceincludes interface definition files associated with source files forthe work, and the source code for shared libraries and dynamicallylinked subprograms that the work is specifically designed to require,such as by intimate data communication or control flow between thosesubprograms and other parts of the work.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The Corresponding Source need not include anything that userscan regenerate automatically from other parts of the CorrespondingSource.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The Corresponding Source for a work in source code form is thatsame work.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section2\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">2</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Basic Permissions.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">All rights granted under this License are granted for the term ofcopyright on the Program, and are irrevocable provided the statedconditions are met.  This License explicitly affirms your unlimitedpermission to run the unmodified Program.  The output from running acovered work is covered by this License only if the output, given itscontent, constitutes a covered work.  This License acknowledges yourrights of fair use or other equivalent, as provided by copyright law.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">You may make, run and propagate covered works that you do notconvey, without conditions so long as your license otherwise remainsin force.  You may convey covered works to others for the sole purposeof having them make modifications exclusively for you, or provide youwith facilities for running those works, provided that you comply withthe terms of this License in conveying all material for which you donot control copyright.  Those thus making or running the covered worksfor you must do so exclusively on your behalf, under your directionand control, on terms that prohibit them from making any copies ofyour copyrighted material outside their relationship with you.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Conveying under any other circumstances is permitted solely underthe conditions stated below.  Sublicensing is not allowed; section 10makes it unnecessary.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section3\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">3</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Protecting Users\' Legal Rights From Anti-Circumvention Law.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">No covered work shall be deemed part of an effective technologicalmeasure under any applicable law fulfilling obligations under article11 of the WIPO copyright treaty adopted on 20 December 1996, orsimilar laws prohibiting or restricting circumvention of suchmeasures.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">When you convey a covered work, you waive any legal power to forbidcircumvention of technological measures to the extent such circumventionis effected by exercising rights under this License with respect tothe covered work, and you disclaim any intention to limit operation ormodification of the work as a means of enforcing, against the work\'susers, your or third parties\' legal rights to forbid circumvention oftechnological measures.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section4\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">4</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Conveying Verbatim Copies.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">You may convey verbatim copies of the Program\'s source code as youreceive it, in any medium, provided that you conspicuously andappropriately publish on each copy an appropriate copyright notice;keep intact all notices stating that this License and anynon-permissive terms added in accord with section 7 apply to the code;keep intact all notices of the absence of any warranty; and give allrecipients a copy of this License along with the Program.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">You may charge any price or no price for each copy that you convey,and you may offer support or warranty protection for a fee.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section5\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">5</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Conveying Modified Source Versions.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">You may convey a work based on the Program, or the modifications toproduce it from the Program, in the form of source code under theterms of section 4, provided that you also meet all of these conditions:</span></p>\n"
"<ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">a) The work must carry prominent notices stating that you modified    it, and giving a relevant date.</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">b) The work must carry prominent notices stating that it is    released under this License and any conditions added under section    7.  This requirement modifies the requirement in section 4 to    “keep intact all notices”.</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">c) You must license the entire work, as a whole, under this    License to anyone who comes into possession of a copy.  This    License will therefore apply, along with any applicable section 7    additional terms, to the whole of the work, and all its parts,    regardless of how they are packaged.  This License gives no    permission to license the work in any other way, but it does not    invalidate such permission if you have separately received it.</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">d) If the work has interactive user interfaces, each must display    Appropriate Legal Notices; however, if the Program has interactive    interfaces that do not display Appropriate Legal Notices, your    work need not make them do so.</li></ul>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">A compilation of a covered work with other separate and independentworks, which are not by their nature extensions of the covered work,and which are not combined with it such as to form a larger program,in or on a volume of a storage or distribution medium, is called an“aggregate” if the compilation and its resulting copyright are notused to limit the access or legal rights of the compilation\'s usersbeyond what the individual works permit.  Inclusion of a covered workin an aggregate does not cause this License to apply to the otherparts of the aggregate.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section6\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">6</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Conveying Non-Source Forms.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">You may convey a covered work in object code form under the termsof sections 4 and 5, provided that you also convey themachine-readable Corresponding Source under the terms of this License,in one of these ways:</span></p>\n"
"<ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">a) Convey the object code in, or embodied in, a physical product    (including a physical distribution medium), accompanied by the    Corresponding Source fixed on a durable physical medium    customarily used for software interchange.</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">b) Convey the object code in, or embodied in, a physical product    (including a physical distribution medium), accompanied by a    written offer, valid for at least three years and valid for as    long as you offer spare parts or customer support for that product    model, to give anyone who possesses the object code either (1) a    copy of the Corresponding Source for all the software in the    product that is covered by this License, on a durable physical    medium customarily used for software interchange, for a price no    more than your reasonable cost of physically performing this    conveying of source, or (2) access to copy the    Corresponding Source from a network server at no charge.</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">c) Convey individual copies of the object code with a copy of the    written offer to provide the Corresponding Source.  This    alternative is allowed only occasionally and noncommercially, and    only if you received the object code with such an offer, in accord    with subsection 6b.</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">d) Convey the object code by offering access from a designated    place (gratis or for a charge), and offer equivalent access to the    Corresponding Source in the same way through the same place at no    further charge.  You need not require recipients to copy the    Corresponding Source along with the object code.  If the place to    copy the object code is a network server, the Corresponding Source    may be on a different server (operated by you or a third party)    that supports equivalent copying facilities, provided you maintain    clear directions next to the object code saying where to find the    Corresponding Source.  Regardless of what server hosts the    Corresponding Source, you remain obligated to ensure that it is    available for as long as needed to satisfy these requirements.</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">e) Convey the object code using peer-to-peer transmission, provided    you inform other peers where the object code and Corresponding    Source of the work are being offered to the general public at no    charge under subsection 6d.</li></ul>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">A separable portion of the object code, whose source code is excludedfrom the Corresponding Source as a System Library, need not beincluded in conveying the object code work.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">A “User Product” is either (1) a “consumer product”, which means anytangible personal property which is normally used for personal, family,or household purposes, or (2) anything designed or sold for incorporationinto a dwelling.  In determining whether a product is a consumer product,doubtful cases shall be resolved in favor of coverage.  For a particularproduct received by a particular user, “normally used” refers to atypical or common use of that class of product, regardless of the statusof the particular user or of the way in which the particular useractually uses, or expects or is expected to use, the product.  A productis a consumer product regardless of whether the product has substantialcommercial, industrial or non-consumer uses, unless such uses representthe only significant mode of use of the product.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">“Installation Information” for a User Product means any methods,procedures, authorization keys, or other information required to installand execute modified versions of a covered work in that User Product froma modified version of its Corresponding Source.  The information mustsuffice to ensure that the continued functioning of the modified objectcode is in no case prevented or interfered with solely becausemodification has been made.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">If you convey an object code work under this section in, or with, orspecifically for use in, a User Product, and the conveying occurs aspart of a transaction in which the right of possession and use of theUser Product is transferred to the recipient in perpetuity or for afixed term (regardless of how the transaction is characterized), theCorresponding Source conveyed under this section must be accompaniedby the Installation Information.  But this requirement does not applyif neither you nor any third party retains the ability to installmodified object code on the User Product (for example, the work hasbeen installed in ROM).</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The requirement to provide Installation Information does not include arequirement to continue to provide support service, warranty, or updatesfor a work that has been modified or installed by the recipient, or forthe User Product in which it has been modified or installed.  Access to anetwork may be denied when the modification itself materially andadversely affects the operation of the network or violates the rules andprotocols for communication across the network.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Corresponding Source conveyed, and Installation Information provided,in accord with this section must be in a format that is publiclydocumented (and with an implementation available to the public insource code form), and must require no special password or key forunpacking, reading or copying.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section7\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">7</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Additional Terms.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">“Additional permissions” are terms that supplement the terms of thisLicense by making exceptions from one or more of its conditions.Additional permissions that are applicable to the entire Program shallbe treated as though they were included in this License, to the extentthat they are valid under applicable law.  If additional permissionsapply only to part of the Program, that part may be used separatelyunder those permissions, but the entire Program remains governed bythis License without regard to the additional permissions.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">When you convey a copy of a covered work, you may at your optionremove any additional permissions from that copy, or from any part ofit.  (Additional permissions may be written to require their ownremoval in certain cases when you modify the work.)  You may placeadditional permissions on material, added by you to a covered work,for which you have or can give appropriate copyright permission.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Notwithstanding any other provision of this License, for material youadd to a covered work, you may (if authorized by the copyright holders ofthat material) supplement the terms of this License with terms:</span></p>\n"
"<ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">a) Disclaiming warranty or limiting liability differently from the    terms of sections 15 and 16 of this License; or</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">b) Requiring preservation of specified reasonable legal notices or    author attributions in that material or in the Appropriate Legal    Notices displayed by works containing it; or</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">c) Prohibiting misrepresentation of the origin of that material, or    requiring that modified versions of such material be marked in    reasonable ways as different from the original version; or</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">d) Limiting the use for publicity purposes of names of licensors or    authors of the material; or</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">e) Declining to grant rights under trademark law for use of some    trade names, trademarks, or service marks; or</li>\n"
"<li style=\" font-family:\'Noto Sans\'; font-size:10pt;\" style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">f) Requiring indemnification of licensors and authors of that    material by anyone who conveys the material (or modified versions of    it) with contractual assumptions of liability to the recipient, for    any liability that these contractual assumptions directly impose on    those licensors and authors.</li></ul>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">All other non-permissive additional terms are considered “furtherrestrictions” within the meaning of section 10.  If the Program as youreceived it, or any part of it, contains a notice stating that it isgoverned by this License along with a term that is a furtherrestriction, you may remove that term.  If a license document containsa further restriction but permits relicensing or conveying under thisLicense, you may add to a covered work material governed by the termsof that license document, provided that the further restriction doesnot survive such relicensing or conveying.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">If you add terms to a covered work in accord with this section, youmust place, in the relevant source files, a statement of theadditional terms that apply to those files, or a notice indicatingwhere to find the applicable terms.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Additional terms, permissive or non-permissive, may be stated in theform of a separately written license, or stated as exceptions;the above requirements apply either way.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section8\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">8</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Termination.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">You may not propagate or modify a covered work except as expresslyprovided under this License.  Any attempt otherwise to propagate ormodify it is void, and will automatically terminate your rights underthis License (including any patent licenses granted under the thirdparagraph of section 11).</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">However, if you cease all violation of this License, then yourlicense from a particular copyright holder is reinstated (a)provisionally, unless and until the copyright holder explicitly andfinally terminates your license, and (b) permanently, if the copyrightholder fails to notify you of the violation by some reasonable meansprior to 60 days after the cessation.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Moreover, your license from a particular copyright holder isreinstated permanently if the copyright holder notifies you of theviolation by some reasonable means, this is the first time you havereceived notice of violation of this License (for any work) from thatcopyright holder, and you cure the violation prior to 30 days afteryour receipt of the notice.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Termination of your rights under this section does not terminate thelicenses of parties who have received copies or rights from you underthis License.  If your rights have been terminated and not permanentlyreinstated, you do not qualify to receive new licenses for the samematerial under section 10.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section9\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">9</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">. Acceptance Not Required for Having Copies.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">You are not required to accept this License in order to receive orrun a copy of the Program.  Ancillary propagation of a covered workoccurring solely as a consequence of using peer-to-peer transmissionto receive a copy likewise does not require acceptance.  However,nothing other than this License grants you permission to propagate ormodify any covered work.  These actions infringe copyright if you donot accept this License.  Therefore, by modifying or propagating acovered work, you indicate your acceptance of this License to do so.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section10\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">0. Automatic Licensing of Downstream Recipients.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Each time you convey a covered work, the recipient automaticallyreceives a license from the original licensors, to run, modify andpropagate that work, subject to this License.  You are not responsiblefor enforcing compliance by third parties with this License.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">An “entity transaction” is a transaction transferring control of anorganization, or substantially all assets of one, or subdividing anorganization, or merging organizations.  If propagation of a coveredwork results from an entity transaction, each party to thattransaction who receives a copy of the work also receives whateverlicenses to the work the party\'s predecessor in interest had or couldgive under the previous paragraph, plus a right to possession of theCorresponding Source of the work from the predecessor in interest, ifthe predecessor has it or can get it with reasonable efforts.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">You may not impose any further restrictions on the exercise of therights granted or affirmed under this License.  For example, you maynot impose a license fee, royalty, or other charge for exercise ofrights granted under this License, and you may not initiate litigation(including a cross-claim or counterclaim in a lawsuit) alleging thatany patent claim is infringed by making, using, selling, offering forsale, or importing the Program or any portion of it.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section11\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1. Patents.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">A “contributor” is a copyright holder who authorizes use under thisLicense of the Program or a work on which the Program is based.  Thework thus licensed is called the contributor\'s “contributor version”.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">A contributor\'s “essential patent claims” are all patent claimsowned or controlled by the contributor, whether already acquired orhereafter acquired, that would be infringed by some manner, permittedby this License, of making, using, or selling its contributor version,but do not include claims that would be infringed only as aconsequence of further modification of the contributor version.  Forpurposes of this definition, “control” includes the right to grantpatent sublicenses in a manner consistent with the requirements ofthis License.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Each contributor grants you a non-exclusive, worldwide, royalty-freepatent license under the contributor\'s essential patent claims, tomake, use, sell, offer for sale, import and otherwise run, modify andpropagate the contents of its contributor version.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">In the following three paragraphs, a “patent license” is any expressagreement or commitment, however denominated, not to enforce a patent(such as an express permission to practice a patent or covenant not tosue for patent infringement).  To “grant” such a patent license to aparty means to make such an agreement or commitment not to enforce apatent against the party.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">If you convey a covered work, knowingly relying on a patent license,and the Corresponding Source of the work is not available for anyoneto copy, free of charge and under the terms of this License, through apublicly available network server or other readily accessible means,then you must either (1) cause the Corresponding Source to be soavailable, or (2) arrange to deprive yourself of the benefit of thepatent license for this particular work, or (3) arrange, in a mannerconsistent with the requirements of this License, to extend the patentlicense to downstream recipients.  “Knowingly relying” means you haveactual knowledge that, but for the patent license, your conveying thecovered work in a country, or your recipient\'s use of the covered workin a country, would infringe one or more identifiable patents in thatcountry that you have reason to believe are valid.  </span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">If, pursuant to or in connection with a single transaction orarrangement, you convey, or propagate by procuring conveyance of, acovered work, and grant a patent license to some of the partiesreceiving the covered work authorizing them to use, propagate, modifyor convey a specific copy of the covered work, then the patent licenseyou grant is automatically extended to all recipients of the coveredwork and works based on it.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">A patent license is “discriminatory” if it does not include withinthe scope of its coverage, prohibits the exercise of, or isconditioned on the non-exercise of one or more of the rights that arespecifically granted under this License.  You may not convey a coveredwork if you are a party to an arrangement with a third party that isin the business of distributing software, under which you make paymentto the third party based on the extent of your activity of conveyingthe work, and under which the third party grants, to any of theparties who would receive the covered work from you, a discriminatorypatent license (a) in connection with copies of the covered workconveyed by you (or copies made from those copies), or (b) primarilyfor and in connection with specific products or compilations thatcontain the covered work, unless you entered into that arrangement,or that patent license was granted, prior to 28 March 2007.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Nothing in this License shall be construed as excluding or limitingany implied license or other defenses to infringement that mayotherwise be available to you under applicable patent law.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section12\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">2. No Surrender of Others\' Freedom.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">If conditions are imposed on you (whether by court order, agreement orotherwise) that contradict the conditions of this License, they do notexcuse you from the conditions of this License.  If you cannot convey acovered work so as to satisfy simultaneously your obligations under thisLicense and any other pertinent obligations, then as a consequence you maynot convey it at all.  For example, if you agree to terms that obligate youto collect a royalty for further conveying from those to whom you conveythe Program, the only way you could satisfy both those terms and thisLicense would be to refrain entirely from conveying the Program.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section13\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">3. Use with the GNU Affero General Public License.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Notwithstanding any other provision of this License, you havepermission to link or combine any covered work with a work licensedunder version 3 of the GNU Affero General Public License into a singlecombined work, and to convey the resulting work.  The terms of thisLicense will continue to apply to the part which is the covered work,but the special requirements of the GNU Affero General Public License,section 13, concerning interaction through a network will apply to thecombination as such.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section14\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">4. Revised Versions of this License.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">The Free Software Foundation may publish revised and/or new versions ofthe GNU General Public License from time to time.  Such new versions willbe similar in spirit to the present version, but may differ in detail toaddress new problems or concerns.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Each version is given a distinguishing version number.  If theProgram specifies that a certain numbered version of the GNU GeneralPublic License “or any later version” applies to it, you have theoption of following the terms and conditions either of that numberedversion or of any later version published by the Free SoftwareFoundation.  If the Program does not specify a version number of theGNU General Public License, you may choose any version ever publishedby the Free Software Foundation.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">If the Program specifies that a proxy can decide which futureversions of the GNU General Public License can be used, that proxy\'spublic statement of acceptance of a version permanently authorizes youto choose that version for the Program.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">Later license versions may give you additional or differentpermissions.  However, no additional obligations are imposed on anyauthor or copyright holder as a result of your choosing to follow alater version.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section15\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">5. Disclaimer of Warranty.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BYAPPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHTHOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM “AS IS” WITHOUT WARRANTYOF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULARPURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAMIS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OFALL NECESSARY SERVICING, REPAIR OR CORRECTION.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section16\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">6. Limitation of Liability.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITINGWILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYSTHE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANYGENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THEUSE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OFDATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRDPARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OFSUCH DAMAGES.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a name=\"section17\"></a><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">1</span><span style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\">7. Interpretation of Sections 15 and 16.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">If the disclaimer of warranty and limitation of liability providedabove cannot be given local legal effect according to their terms,reviewing courts shall apply local law that most closely approximatesan absolute waiver of all civil liability in connection with theProgram, unless a warranty or assumption of liability accompanies acopy of the Program in return for a fee.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Noto Sans\'; font-size:10pt;\">END OF TERMS AND CONDITIONS</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'Noto Sans\'; font-size:10pt; font-weight:600;\"><br /></p></body></html>")
        self.textGPL.setOpenExternalLinks(True)
        self.textGPL.setObjectName("textGPL")
        self.verticalLayout_2.addWidget(self.textGPL)
        self.tabWidget.addTab(self.tabGPL, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(AboutDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(AboutDialog)
        self.tabWidget.setCurrentIndex(2)
        self.buttonBox.accepted.connect(AboutDialog.accept)
        self.buttonBox.rejected.connect(AboutDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(AboutDialog)

    def retranslateUi(self, AboutDialog):
        _translate = QtCore.QCoreApplication.translate
        AboutDialog.setWindowTitle(_translate("AboutDialog", "About PyCirkuit", "Dialog Title"))
        self.textAbout.setHtml(_translate("AboutDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:12pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-weight:600;\">PyCirkuit</span><span style=\" font-size:11pt;\"> is a GUI front-end for </span><a href=\"https://ece.uwaterloo.ca/~aplevich/Circuit_macros/\"><span style=\" font-size:11pt; text-decoration: underline; color:#2980b9;\">Circuit Macros</span></a><span style=\" font-size:11pt;\"> and </span><a href=\"https://gitlab.com/aplevich/dpic\"><span style=\" text-decoration: underline; color:#2980b9;\">Dpic</span></a><span style=\" font-size:11pt;\"> by Dwight Aplevich, which make up a software chain for drawing high-quality line diagrams to be included in TeX, LaTeX, LibreOffice, web or similar documents. PyCirkuit is written in Python 3 using the PyQt5 libraries, but is largely inspired (in both ideas and code snippets) on </span><a href=\"https://wwwu.uni-klu.ac.at/magostin/cirkuit.html\"><span style=\" font-size:11pt; text-decoration: underline; color:#2980b9;\">Cirkuit</span></a><span style=\" font-size:11pt;\">, a C++ application written by Matteo Agostinelli using KDE4 libraries.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt;\">PyCirkuit compiles the source code on demand to generate a live preview, and can export the resulting image in various formats to be included directly in your documents.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt;\">You can send bug reports via the </span><a href=\"https://github.com/orestesmas/pycirkuit\"><span style=\" font-size:11pt; text-decoration: underline; color:#2980b9;\">project repository on GitHub</span></a><span style=\" font-size:11pt;\">.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt;\">If you are willing to contribute, please read </span><a href=\"https://github.com/orestesmas/pycirkuit/blob/master/CONTRIBUTING.md\"><span style=\" font-size:11pt; text-decoration: underline; color:#2980b9;\">these guidelines</span></a><span style=\" font-size:11pt;\">.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt;\">PyCirkuit uses icons from KDE\'s &quot;Oxygen&quot; set, which are under the LGPL license. See </span><a href=\"https://techbase.kde.org/Projects/Oxygen/Licensing\"><span style=\" font-size:11pt; text-decoration: underline; color:#2980b9;\">https://techbase.kde.org/Projects/Oxygen/Licensing</span></a><span style=\" font-size:11pt;\">.</span></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabAbout), _translate("AboutDialog", "&About", "Tab text"))
        self.textVersion.setHtml(_translate("AboutDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:12pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-weight:600;\">Version {versionNumber}</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt;\">Using:</span></p>\n"
"<ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px; margin-right: 0px; -qt-list-indent: 1;\"><li style=\" font-size:11pt;\" style=\" margin-top:12px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://www.riverbankcomputing.com/software/pyqt/intro\"><span style=\" text-decoration: underline; color:#2980b9;\">PyQt5</span></a> Python bindings to Qt5 by Riverbank Computing.</li>\n"
"<li style=\" font-size:11pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://ece.uwaterloo.ca/~aplevich/Circuit_macros/\"><span style=\" text-decoration: underline; color:#2980b9;\">Circuit Macros</span></a>, by Dwight Aplevich.</li>\n"
"<li style=\" font-size:11pt;\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://www.gnu.org/software/m4/\"><span style=\" text-decoration: underline; color:#2980b9;\">GNU M4</span></a> Macro Processor, from the GNU project.</li>\n"
"<li style=\" font-size:11pt;\" style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://ece.uwaterloo.ca/~aplevich/dpic\"><span style=\" text-decoration: underline; color:#2980b9;\">Dpic</span></a>, by Dwight Aplevich.</li>\n"
"<li style=\" font-size:11pt;\" style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://ctan.org/\"><span style=\" text-decoration: underline; color:#2980b9;\">LaTeX + TIkZ</span></a>, by Donald Knuth, Leslie Lamport, Till Tantau and many more contributors.</li>\n"
"<li style=\" font-size:11pt;\" style=\" margin-top:0px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><a href=\"https://linux.die.net/man/1/pdftoppm\"><span style=\" text-decoration: underline; color:#2980b9;\">PdfToPpm</span></a>, a software by <a href=\"https://poppler.freedesktop.org/\"><span style=\" text-decoration: underline; color:#2980b9;\">The Poppler project</span></a>. </li></ul></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabVersion), _translate("AboutDialog", "&Version", "Tab text"))
        self.textAuthor.setHtml(_translate("AboutDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:12pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-weight:600;\">Orestes Mas</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-style:italic;\">Original Author, Maintainer and Translator to Catalan</span></p>\n"
"<hr />\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-weight:600;\">Aniol Marti</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-style:italic;\">Debian packages for PyCirkuit, Dpic and Circuit Macros, beta-testing and bug reporting. Contribution to CLI code.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:11pt; font-style:italic;\"><br /></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-weight:600;\">Guillem Casadesus</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-style:italic;\">Beta-testing and bug reporting on macOS</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:11pt; font-style:italic;\"><br /></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-weight:600;\">Billy Andriamahazomandimby</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-style:italic;\">Beta-testing and bug reporting on Windows</span></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabAuthor), _translate("AboutDialog", "A&uthors", "Tab text"))
        self.textLicense.setHtml(_translate("AboutDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:12pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-weight:600;\">PyCirkuit</span><span style=\" font-size:11pt;\"> is {copyrightInfo} &lt;orestes@tsc.upc.edu&gt;</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt;\">You may use, distribute and copy PyCirkuit under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, which is shown below, or (at your option) any later version.</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt;\">PyCirkuit uses also some icons and other artwork from &quot;Oxygen&quot; set, from the KDE project. These icons are under the GNU LGPL v3 license. See </span><a href=\"https://techbase.kde.org/Projects/Oxygen/Licensing\"><span style=\" font-size:11pt; text-decoration: underline; color:#2980b9;\">https://techbase.kde.org/Projects/Oxygen/Licensing</span></a><span style=\" font-size:11pt;\"> and </span><a href=\"http://www.gnu.org/copyleft/lesser.html\"><span style=\" font-size:11pt; text-decoration: underline; color:#2980b9;\">http://www.gnu.org/copyleft/lesser.html</span></a></p></body></html>"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabGPL), _translate("AboutDialog", "&License Agreement"))


from pycirkuit.resources import resources_rc


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    AboutDialog = QtWidgets.QDialog()
    ui = Ui_AboutDialog()
    ui.setupUi(AboutDialog)
    AboutDialog.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-
"""
Module implementing a customized TextEditor
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Third-party imports
from PyQt5.QtCore import QCoreApplication, Qt
from PyQt5.QtWidgets import QTextEdit
from PyQt5.QtGui import QKeySequence

# Translation function
_translate = QCoreApplication.translate

class pycktTextEditor(QTextEdit):
    """
    Class documentation goes here.
    """
    def __init__(self, parent=None):
        """
        Constructor
        
        @param parent reference to the parent widget
        @type QTextEdit
        """
        super().__init__(parent)

    def keyPressEvent(self, event):
        if event.matches(QKeySequence.ZoomIn):
            self._change_font_size(1)
        elif event.matches(QKeySequence.ZoomOut):
            self._change_font_size(-1)
        else:
            super().keyPressEvent(event)
    
    def wheelEvent(self, event):
        if (event.modifiers()==Qt.ControlModifier):
            event.accept()
            numSteps = event.angleDelta() / 120
            self._change_font_size(numSteps.y())
        else:
            super().wheelEvent(event)
        
    def _change_font_size(self, steps):
        editorFont = self.font()
        fontSize = editorFont.pointSize()
        newFontSize = fontSize + steps
        editorFont.setPointSize(newFontSize)
        self.setFont(editorFont)



# -*- coding: utf-8 -*-
"""
Module implementing a Text Highlighter
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Third-party imports
from PyQt5.QtCore import QRegExp
from PyQt5.QtGui import QFont, QSyntaxHighlighter, QTextCharFormat,  QColor


class PyCirkuitHighlighter(QSyntaxHighlighter):
    def __init__(self, parent=None):
        super().__init__(parent)

        # Variable to hold the formatting rules for each category
        self.highlightingRules = []

        # These are the keywords and patterns to be highlighted, categorized.

        # Formatting rules for punctuation signs
        punctuationFormat = QTextCharFormat()
        punctuationFormat.setFontWeight(QFont.Bold)
        punctuationFormat.setForeground(QColor('sienna'))
        self.highlightingRules.append((QRegExp("[()\{\},;]"), punctuationFormat))
 
        # Formatting rules for operators
        operatorFormat = QTextCharFormat()
        operatorFormat.setFontItalic(True)
        operatorFormat.setForeground(QColor('teal'))
        self.highlightingRules.append((QRegExp("(\+|\-|\*|/|\.|=|<-|->|<->){1,1}"), operatorFormat))
        
        # Label displaying rules
        labelFormat = QTextCharFormat()
        labelFormat.setFontWeight(QFont.Bold)
        labelFormat.setForeground(QColor('deeppink'))
        self.highlightingRules.append((QRegExp("[A-Z]+[A-Za-z0-9]*:"), labelFormat))

        # String displaying rules
        stringFormat = QTextCharFormat()
        stringFormat.setFontWeight(QFont.Bold)
        stringFormat.setForeground(QColor('firebrick'))
        self.highlightingRules.append((QRegExp('\".*\"'), stringFormat))

        # Now the beggining and ending PIC commands in red
        picBoundaryPatterns = ["^\.P[SE]{1,1}\\b"]
        picBoundaryFormat = QTextCharFormat()
        picBoundaryFormat.setForeground(QColor('darkred'))
        self.highlightingRules.extend([(QRegExp(pattern), picBoundaryFormat)
                for pattern in picBoundaryPatterns])
    
        # Format for PIC commands: Italic Blue
        picPatterns = ["\\b1st\\b","\\b2nd\\b","\\b3rd\\b","\\b4th\\b","\\b5th\\b","\\b6th\\b",
        "\\b7th\\b","\\b8th\\b","\\b9th\\b","\\bHere\\b","\\babove\\b","\\band\\b",
        "\\barc\\b","\\barrow\\b","\\barrowhead\\b","\\barrowht\\b","\\barrowwid\\b","\\bat\\b",
        "\\batan2\\b","\\bbelow\\b","\\bbetween\\b","\\bbox\\b","\\bby\\b","\\bccw\\b","\\.center\\b", 
        "\\bchop\\b","\\bcircle\\b","\\bcolor\\b","\\bcolour\\b","\\bcolored\\b","\\bcoloured\\b",
        "\\bcommand\\b","\\bcos\\b","\\bcw\\b","\\bdashed\\b","\\bdiam\\b","\\bdo\\b",
        "\\bdotted\\b","\\bdown\\b","\\bellipse\\b","\\belse\\b","\\.end\\b","\\bexp\\b",
        "\\bfill\\b","\\bfor\\b","\\bfrom\\b","\\bheight\\b","\\bht\\b","\\bif\\b",
        "\\binvis(?:ible)?\\b","\\blast\\b","\\bleft\\b","\\bline\\b","\\bljust\\b","\\bmax\\b",
        "\\bmin\\b","\\bmove\\b","\\bof the way between\\b","\\boutline\\b","\\boutlined\\b",
        "\\brad\\b","\\breset\\b","\\bright\\b","\\brjust\\b","\\bshaded\\b","\\bsin\\b",
        "\\bspline\\b","\\bsprintf\\b","\\bsqrt\\b","\\.start\\b","\\btan\\b","\\bthen\\b","\\bthick\\b",
        "\\bto\\b","\\bup\\b","\\bwid\\b","\\bwidth\\b","\\bwith\\b","\\.n\\b","\\.s\\b",
        "\\.e\\b","\\.w\\b","\\.ne\\b","\\.nw\\b","\\.se\\b","\\.sw\\b"]
        picFormat = QTextCharFormat()
        picFormat.setFontItalic(True)
        picFormat.setForeground(QColor('blue'))
        self.highlightingRules.extend([(QRegExp(pattern), picFormat) for pattern in picPatterns])

        # Format for PIC variables: Italic Red
        picVariables = ["\\blinewid\\b","\\blinethick\\b","\\barcrad\\b","\\bcirclerad\\b","\\bellipseht\\b",
        "\\bellipsewid\\b","\\bmoveht\\b","\\bmovewid\\b","\\btextht\\b","\\btextwid\\b","\\barrowht\\b",
        "\\barrowwid\\b","\\barrowhead\\b","\\bdashwid\\b","\\bmaxpswid\\b","\\bmaxpsht\\b","\\bscale\\b",
        "\\bfillval\\b"]
        picFormat = QTextCharFormat()
        picFormat.setFontItalic(True)
        picFormat.setForeground(QColor('red'))
        self.highlightingRules.extend([(QRegExp(pattern), picFormat) for pattern in picVariables])

        # Commands and format for M4 primitives
        m4Patterns = ["\\bchangequote\\b","\\bdefine\\b","\\bdivert\\b","\\bdivnum\\b","\\bdnl\\b","\\bdumpdef\\b",
        "\\berrprint\\b","\\beval\\b","\\bifdef\\b","\\bifelse\\b","\\binclude\\b","\\bincr\\b",
        "\\bindex\\b","\\blen\\b","\\bmaketemp\\b","\\bsinclude\\b","\\bsubstr\\b","\\bsyscmd\\b",
        "\\btranslit\\b","\\bundefine\\b","\\bundivert\\b",]
        m4Format = QTextCharFormat()
        m4Format.setFontWeight(QFont.Bold)
        m4Format.setForeground(QColor('DarkOrchid'))
        self.highlightingRules.extend([(QRegExp(pattern), m4Format) for pattern in m4Patterns])
        
        cmPatterns = ["\\bAND_gate\\b","\\bAND_gen\\b","\\bAND_ht\\b","\\bAND_wd\\b","\\bBOX_gate\\b","\\bBUFFER_gate\\b",
        "\\bBUFFER_gen\\b","\\bBUF_ht\\b","\\bBUF_wd\\b","\\bCos\\b","\\bCosine\\b","\\bDarlington\\b",
        "\\bE_\\b","\\bEquidist3\\b","\\bFF_ht\\b","\\bFF_wid\\b","\\bFector\\b","\\bFlipFlop\\b",
        "\\bFlipFlop6\\b","\\bFlipFlopJK\\b","\\bG_hht_\\b","\\bHOMELIB_\\b","\\bH_ht\\b","\\bInt_\\b",
        "\\bIOdefs\\b","\\bIntersect_\\b","\\bLH_symbol\\b","\\bLoopover_\\b","\\bLT_symbol\\b","\\bL_unit\\b",
        "\\bMax\\b","\\bMin\\b","\\bMux\\b","\\bMux_ht\\b","\\bMux_wid\\b","\\bMx_pins\\b",
        "\\bNAND_gate\\b","\\bNOR_gate\\b","\\bNOT_circle\\b","\\bNOT_gate\\b","\\bNOT_rad\\b","\\bNXOR_gate\\b",
        "\\bN_diam\\b","\\bN_rad\\b","\\bOR_gate\\b","\\bOR_gen\\b","\\bOR_rad\\b","\\bPoint_\\b",
        "\\bRect_\\b","\\bSin\\b","\\bView3D\\b","\\bVperp\\b","\\bXOR_gate\\b","\\bXOR_off\\b",
        "\\babove_\\b","\\babs_\\b","\\badc\\b","\\bamp\\b","\\balong_\\b","\\bantenna\\b",
        "\\barca\\b","\\barcd\\b","\\barcr\\b","\\barcto\\b","\\barrowline\\b","\\bb_\\b",
        "\\bb_current\\b","\\bbattery\\b","\\bbeginshade\\b","\\bbell\\b","\\bbelow_\\b","\\bbi_tr\\b",
        "\\bbi_trans\\b","\\bboxcoord\\b","\\bboxdim\\b","\\bbp__\\b","\\bbswitch\\b","\\bbuzzer\\b",
        "\\bc_fet\\b","\\bcapacitor\\b","\\bcbreaker\\b","\\bcct_init\\b","\\bcintersect\\b","\\bclabel\\b",
        "\\bconsource\\b","\\bcontact\\b","\\bcontline\\b","\\bcorner\\b","\\bcosd\\b","\\bcross\\b","\\bcross3D\\b",
        "\\bcrossover\\b","\\bcrosswd_\\b","\\bcsdim_\\b","\\bdac\\b","\\bd_fet\\b","\\bdabove\\b",
        "\\bdarrow\\b","\\bdarrow_init\\b","\\bdashline\\b","\\bdbelow\\b","\\bdcosine3D\\b","\\bdef_bisect\\b",
        "\\bdelay\\b","\\bdelay_rad_\\b","\\bdeleminit_\\b","\\bdend\\b","\\bdiff3D\\b","\\bdiff_\\b",
        "\\bdimen_\\b","\\bdimension_\\b","\\bdiode\\b","\\bdir_\\b","\\bdistance\\b","\\bdirection_\\b",
        "\\bdistance\\b","\\bdlabel\\b","\\bdleft\\b","\\bdline\\b","\\bdlinewid\\b","\\bdn_\\b",
        "\\bdljust\\b","\\bdn_\\b","\\bdna_\\b","\\bdnm_\\b","\\bdot\\b","\\bdot3D\\b",
        "\\bdotrad_\\b","\\bdown_\\b","\\bdright\\b","\\bdrjust\\b","\\bdswitch\\b","\\bdtee\\b",
        "\\bdtor_\\b","\\bdturn\\b","\\be_\\b","\\be_fet\\b","\\bearphone\\b","\\bebox\\b",
        "\\belchop\\b","\\beleminit_\\b","\\belen_\\b","\\bem_arrows\\b","\\bendshade\\b","\\bexpe\\b",
        "\\bf_box\\b","\\bfill_\\b","\\bfitcurve\\b","\\bfor_\\b","\\bfuse\\b","\\bg_\\b",
        "\\bgap\\b","\\bgen_init\\b","\\bglabel_\\b","\\bgpar_\\b","\\bgpolyline_\\b","\\bgrid_\\b",
        "\\bground\\b","\\bgshade\\b","\\bhoprad_\\b","\\bht_\\b","\\bifdpic\\b","\\bifgpic\\b",
        "\\bifinstr\\b","\\bifmfpic\\b","\\bifmpost\\b","\\bifpgf\\b","\\bifpostscript\\b","\\bifpstricks\\b",
        "\\bifroff\\b","\\bifxfig\\b","\\bigbt\\b","\\bin__\\b","\\binductor\\b","\\binner_prod\\b",
        "\\bintegrator\\b","\\bintersect_\\b","\\bj_fet\\b","\\blarrow\\b","\\blbox\\b","\\bleft_\\b",
        "\\blength3D\\b","\\blg_bartxt\\b","\\blg_pin\\b","\\blg_pintxt\\b","\\blg_plen\\b","\\blin_leng\\b",
        "\\blinethick_\\b","\\bljust_\\b","\\bllabel\\b","\\bloc_\\b","\\blog10E_\\b","\\blog_init\\b",
        "\\bloge\\b","\\blp_xy\\b","\\blpop\\b","\\blswitch\\b","\\blt_\\b","\\bmanhattan\\b",
        "\\bmicrophone\\b","\\bmm__\\b","\\bmosfet\\b","\\bm4xpand\\b","\\bm4lstring\\b","\\bm4_arrow\\b",
        "\\bm4xtract\\b","\\bn_\\b","\\bne_\\b","\\bneg_\\b","\\bnport\\b","\\bnw_\\b",
        "\\bopamp\\b","\\bopen_arrow\\b","\\bpar_\\b","\\bpc__\\b","\\bpi_\\b","\\bpmod\\b",
        "\\bpoint_\\b","\\bpolar_\\b","\\bpotentiometer\\b","\\bprint3D\\b","\\bprod_\\b","\\bproject\\b",
        "\\bpsset_\\b","\\bpt__\\b","\\br_\\b","\\brarrow\\b","\\brect_\\b","\\brelay\\b",
        "\\bresetrgb\\b","\\bresistor\\b","\\breversed\\b","\\brgbdraw\\b","\\brgbfill\\b","\\bright_\\b",
        "\\brjust_\\b","\\brlabel\\b","\\brot3Dx\\b","\\brot3Dy\\b","\\brot3Dz\\b","\\brpoint_\\b",
        "\\brpos_\\b","\\brrot_\\b","\\brs_box\\b","\\brsvec_\\b","\\brt_\\b","\\brtod_\\b",
        "\\brtod__\\b","\\brvec_\\b","\\bs_\\b","\\bs_box\\b","\\bs_dp\\b","\\bs_ht\\b",
        "\\bs_init\\b","\\bs_name\\b","\\bs_wd\\b","\\bsc_draw\\b","\\bse_\\b","\\bsetrgb\\b",
        "\\bsetview\\b","\\bsfg_init\\b","\\bsfgabove\\b","\\bsfgarc\\b","\\bsfgbelow\\b","\\bsfgline\\b",
        "\\bsfgnode\\b","\\bsfgself\\b","\\bshade\\b","\\bshadebox\\b","\\bsign_\\b","\\bsinc\\b",
        "\\bsind\\b","\\bsinusoid\\b","\\bsource\\b","\\bsourcerad_\\b","\\bsp_\\b","\\bspeaker\\b",
        "\\bsprod3D\\b","\\bsum3D\\b","\\bsum_\\b","\\bsvec_\\b","\\bsw_\\b","\\bswitch\\b",
        "\\bta_xy\\b","\\bthicklines_\\b","\\bthinlines_\\b","\\bthyristor\\b","\\btline\\b","\\btr_xy\\b",
        "\\btr_xy_init\\b","\\btransformer\\b","\\bttmotor\\b","\\btwopi_\\b","\\bujt\\b","\\bunit3D\\b",
        "\\bup_\\b","\\bup__\\b","\\bvariable\\b","\\bvec_\\b","\\bvlength\\b","\\bvperp\\b",
        "\\bvrot_\\b","\\bvscal_\\b","\\bw_\\b","\\bwid_\\b","\\bwinding\\b","\\bxtal\\b",
        "\\bxtract\\b",]
        cmFormat = QTextCharFormat()
        cmFormat.setFontWeight(QFont.Bold)
        cmFormat.setForeground(QColor('DarkGreen'))
        self.highlightingRules.extend([(QRegExp(pattern), cmFormat) for pattern in cmPatterns])

        # Comments are displayed in gray
        singleLineCommentFormat = QTextCharFormat()
        singleLineCommentFormat.setForeground(QColor('DarkGray'))
        self.highlightingRules.append((QRegExp("#[^\n]*"), singleLineCommentFormat))


    def highlightBlock(self, text):
        for pattern, format in self.highlightingRules:
            expression = QRegExp(pattern)
            index = expression.indexIn(text)
            while index >= 0:
                length = expression.matchedLength()
                self.setFormat(index, length, format)
                index = expression.indexIn(text, index + length)





# -*- coding: utf-8 -*-
"""
Module implementing a class to handle the pdftoppm external tool
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#


# Third-party imports
from PyQt5.QtCore import QCoreApplication

# Local application imports
from pycirkuit.tools.tool_base import ExternalTool

# Translation function
_translate = QCoreApplication.translate

class ToolPdfToPng(ExternalTool):
    # Class variable
    ID = 'PDFTOPNG'
    def __init__(self):
        super().__init__("pdftoppm", _translate("ExternalTool", "PDF to PNG image converter", "Tool Long Name"))
        
    def execute(self, baseName, resolution=150):
        # Calculate src and dst names
        src = baseName + '.pdf'
        dst = baseName + '.png'
        # Instantiate a settings object to load config values. At this point the config have valid entries, so don't test much
        command = [self.executableName, "-png", "-r", "{ppp}".format(ppp=resolution), "{source}".format(source=src)]
        errMsg = _translate("ExternalTool", "PDFTOPPM: Error converting PDF -> PNG", "Error message")
        super().execute(command, errMsg, destination=dst)



# -*- coding: utf-8 -*-
"""
Module implementing pycktImageViewer
"""
# Copyright (C) 2018 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#

# Standard library imports
import os
from math import log10

# Third-party imports
from PyQt5.QtCore import QCoreApplication, Qt, QRectF, QPointF, pyqtSignal
from PyQt5.QtWidgets import QGraphicsScene, QGraphicsView
from PyQt5.QtGui import QPixmap, QFont

# Local application imports
import pycirkuit
from pycirkuit.exceptions import *
from pycirkuit.tools.pdftopng import ToolPdfToPng

# Translation function
_translate = QCoreApplication.translate

class pycktImageViewer(QGraphicsView):
    """
    A viewer for images compiled from circuit macros source files.
    Being a subclass of QGraphicsView, this class handles zoom using
    the mouse wheel.
    """
    # A signal to inform of errors
    conversion_failed = pyqtSignal(PyCirkuitError)
    image_changed = pyqtSignal()

    def __init__(self, parent = None, maxZoom = 5):
        """
        Constructor

        @param parent reference to the parent widget
        @type QWidget
        """
        super().__init__(parent)
        
        # Initialize the graphics Scene-View pair with an empty pixmap
        # First the scene...
        self.__scene = QGraphicsScene(self)
        # Initial image does not correspond to any file
        self.__file_base_name = None
        # Then the pixmap (initially empty). We maintain a reference to the pixmap, for easy access
        self.__pixmap_item = self.__scene.addPixmap(QPixmap())
        # Then the graphics view (ourselves)
        self.setScene(self.__scene)

        # Set operation mode (some options set up in Designer)
        self.setAlignment(Qt.AlignCenter)   # Image appears centered if fits in view
        
        # Zoom parameters initialization
        self.__ppi_base = 150      # Base image resolution (pixels per inch)
        self.__max_zoom = maxZoom  # Maximum magnification factor (default = 5)
        self.__max_steps = 10      # No. of wheel steps required to achieve max. zoom
        self.__wheel_steps = 0     # Initial value for wheel position
        self.__current_ppi = self.__calc_ppi(self.__wheel_steps)  # Initial value for current resolution
    
    def __adjust_view(self):
        self.setSceneRect(self.__scene.itemsBoundingRect())

    def __calc_ppi(self, wheelSteps):
        """
        This function calculates the image resolution required to achieve the desired zoom level,
        indicated by number of steps the mouse wheel has turned.
        
        @param wheelSteps Number of steps the user has turned the mouse wheel (must be bounded)
        @type integer
        """
        # 'ratio' could be precalculated in __init__. Doing so here is a waste of cpu cycles
        # but allows for more (future) flexibility, as we can, for instance, use variable ratios.
        ratio = 10 ** (log10(self.__max_zoom)/self.__max_steps)
        newPPI = round(self.__ppi_base * (ratio ** wheelSteps))
        return newPPI

    def __clear_items(self):
        for item in self.__scene.items():
            self.__scene.removeItem(item)

    def __qBound(self, minVal, current, maxVal):
        """
        PyQt5 does not wrap the qBound function from Qt's global namespace.
        This is equivalent. Returns "current" if it's between "minVal" and "maxVal",
        otherwise saturates to one of these two limits.
        """
        return max(min(current, maxVal), minVal)

    def clearImage(self):
        self.__clear_items()
        self.__pixmap_item = self.__scene.addPixmap(QPixmap())
        self.__adjust_view()

    def getRect(self):
        return self.__pixmap_item.pixmap().rect()

    def setImage(self, fileBaseName, adjustIGU=False):
        """
        Tries to load a PNG image from disk and display it
        
        Args:
            fileBaseName (str): the file name of the image to load (without extension) relative to current dir.
            adjustIGU (bool): Whether we want to adjust the size of IGU viewport to fit the image or not (default=False).
        """
        self.__clear_items()
        try:
            newPixmap = QPixmap("{baseName}.png".format(baseName=fileBaseName))
            if newPixmap.isNull():
                raise PyCirkuitError(
                    _translate("PyCirkuitError", "The converted image could not be loaded.", "Exception message"), 
                    _translate("PyCirkuitError", "PyCirkuit Internal Error", "Exception title"), 
                    moreInfo = _translate("PyCirkuitError", "This error should not happen in a normal operation. Please contact the developers.", "Exception additional info")
                )
        except PyCirkuitError as err:
            self.setText( _translate("PyCirkuitError", "Internal error: cannot load image", "Displayed error message"))
            self.conversion_failed.emit(err)
        else:
            self.__file_base_name = fileBaseName
            self.__pixmap_item = self.__scene.addPixmap(newPixmap)
            if adjustIGU:
                self.image_changed.emit()
            self.__adjust_view()

    def setText(self, newText):
        """
        Displays a text string (in bigger font and in red color) into the viewport
        
        Args:
            newText (str): the string to display.
        """       
        self.__clear_items()
        font = QFont()
        font.setPointSize(22)
        self.__scene.addText(newText, font).setDefaultTextColor(Qt.red)
        self.__adjust_view()

    def wheelEvent(self,event):
        if (self.__file_base_name != None) and (event.modifiers()==Qt.ControlModifier):
            event.accept()
            # Accumulated mouse wheel steps, bounded to certain limits
            self.__wheel_steps = self.__qBound(
                -self.__max_steps,
                 self.__wheel_steps + event.angleDelta().y() / 120,
                 self.__max_steps)
            newPPI = self.__calc_ppi(self.__wheel_steps)
            # Do nothing if reached one of the zoom limits
            if (newPPI == self.__current_ppi):
                return
            
            # Get the cursor's position and translate to pixmap coordinates
            scenePos = self.mapToScene(event.pos())
            # Now find the % offset from pixmap's origin. This doesn't change with zoom
            percentX = scenePos.x()/self.__scene.itemsBoundingRect().width()
            percentY = scenePos.y()/self.__scene.itemsBoundingRect().height()
            
            # Now perform the zoom by converting from PDF with suitable resolution
            # Set working dir
            saveWD = os.getcwd()
            os.chdir(pycirkuit.__tmpDir__.path())
            try:
                converter = ToolPdfToPng()
                converter.execute(self.__file_base_name, resolution = newPPI)
                self.setImage(self.__file_base_name, adjustIGU=True)
                
                # If Scene contents area is smaller than current viewport size, center scene in view
                currentSceneRect = self.__scene.itemsBoundingRect()
                currentViewportRect = QRectF(self.rect())
                if (currentViewportRect.contains(currentSceneRect)):
                    self.setSceneRect(currentSceneRect)
                    self.centerOn(currentSceneRect.center())
                # Else, keep scene point under cursor at the same viewport position
                else:
                    # Calculate the scene coordinates of the position which is (%X,%Y) from origin
                    newScenePos = QPointF(currentSceneRect.width()*percentX, currentSceneRect.height()*percentY)
                    # Calculate pixel offset between mouse position and viewport center
                    offset = QPointF(self.width()/2, self.height()/2) - QPointF(event.pos())
                    # Then adjust things so that this scene position appears under the mouse
                    self.centerOn(newScenePos +offset)
            except PyCirkuitError as err:
                self.setText(_translate("PyCirkuitError", "Error creating zoomed image.", "Displayed error message"))
                self.conversion_failed.emit(err)
            else:
                self.__current_ppi = newPPI
            finally:
                os.chdir(saveWD)
        else:
            event.ignore()



# -*- coding: utf-8 -*-
"""
Module storing the application version number and other metadata in a single place
"""
# Copyright (C) 2018-2019 Orestes Mas
# This file is part of PyCirkuit.
#
# PyCirkuit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyCirkuit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyCirkuit.  If not, see <https://www.gnu.org/licenses/>.
#
from enum import Enum

__all__ = ["mainwindow"]

__productname__   = "PyCirkuit"
# Expecting trailing "-rcN" or "" for stable releases.
__version__       = "0.5.0"
__description__   = "A front-end for Circuit Macros"
__author__        = "Orestes Mas"
__author_email__  = "orestes@tsc.upc.edu"
__license_short__ = "GPLv3+"
__license_long__  = "Licensed under the GNU GPL v3 or any later version"
__copyright__     = "Copyright 2018-2019 {__author__}".format(**locals())
__bigcopyright__  = """{__productname__} {__version__}
{__license_long__}""".format(**locals())
__homepage__      = "https://github.com/orestesmas/pycirkuit"

# The temporary working dir should be globally addressed through the entire application
__tmpDir__ = None

# Global variable that indicates if we are being executed on a desktop environment or not
__haveGUI__ = None

# CLI options
class Option(Enum):
    TIKZ = 'tikzOption'
    PDF = 'pdfOption'
    PNG = 'pngOption'
    JPEG = 'jpgOption'
    SVG = 'svgOption'
    DPI = 'dpiOption'
    QUAL = 'qualityOption'
    REC = 'recurseOption'
    LINK = 'followLinksOption'
    DEST = 'destDirOption'
    OVER = 'overwriteOption'

